@@ -82,13 +82,8 @@ where
     crate::tests::Tester::run(|tester| {
         let module = op(tester)?;
 
-        match ::std::env::var("PRINT_HYGIENE") {
-            Ok(ref s) if s == "1" => {
-                let hygiene_src = tester.print(&module.clone().fold_with(&mut HygieneVisualizer));
-                println!("----- Hygiene -----\n{}", hygiene_src);
-            }
-            _ => {}
-        }
+        let hygiene_src = tester.print(&module.clone().fold_with(&mut HygieneVisualizer));
+        println!("----- Hygiene -----\n{}", hygiene_src);
 
         let module = module.fold_with(&mut hygiene_with_config(config));
 
@@ -102,6 +97,9 @@ where
         };
 
         if actual != expected {
+            println!("----- Actual -----\n{}", actual);
+            println!("----- Diff -----");
+
             assert_eq!(DebugUsingDisplay(&*actual), DebugUsingDisplay(&*expected));
         }
 
@@ -1117,13 +1115,15 @@ fn issue_295_01() {
                 )?
                 .fold_with(&mut OnceMarker::new(&[("bar", &[mark1, mark1, mark1])])))
         },
-        "export const bar = {};
+        "
+        export const bar = {
+        };
         class Foo {
-
-            constructor() {
+            constructor(){
                 bar;
             }
-        }",
+        }
+        ",
         Default::default(),
     );
 }
@@ -1148,14 +1148,15 @@ fn issue_295_02() {
                 )?
                 .fold_with(&mut OnceMarker::new(&[("bar", &[mark1, mark2])])))
         },
-        "const bar1 = {};
-        export { bar1 as bar };
+        "
+        export const bar = {
+        };
         class Foo {
-
-            constructor() {
+            constructor(){
                 bar;
             }
-        }",
+        }
+        ",
         Default::default(),
     );
 }
@@ -1405,7 +1406,7 @@ fn opt_4() {
                     }
                     ",
                 )?
-                .fold_with(&mut OnceMarker::new(&[("a", &[mark1, mark2, mark1])]));
+                .fold_with(&mut OnceMarker::new(&[("a", &[mark1, mark2, mark2])]));
             Ok(stmts)
         },
         "
@@ -1442,7 +1443,7 @@ fn opt_5() {
                     }
                     ",
                 )?
-                .fold_with(&mut OnceMarker::new(&[("a", &[mark1, mark2, mark1])]));
+                .fold_with(&mut OnceMarker::new(&[("a", &[mark1, mark2, mark2])]));
             Ok(stmts)
         },
         "
@@ -1501,3 +1502,43 @@ fn opt_6() {
         ",
     );
 }
+
+#[test]
+fn issue_2211() {
+    test(
+        |tester| {
+            let mark1 = Mark::fresh(Mark::root());
+            let mark2 = Mark::fresh(Mark::root());
+
+            let stmts = tester
+                .parse_stmts(
+                    "actual1.js",
+                    "
+                    var _bar = require('./bar');
+                    const makeX = ()=>{
+                        const _bar = ()=>(0, _bar).bar()
+                        ;
+                        return {
+                            _bar
+                        };
+                    };
+                    ",
+                )?
+                .fold_with(&mut OnceMarker::new(&[(
+                    "_bar",
+                    &[mark1, mark2, mark1, mark2],
+                )]));
+            Ok(stmts)
+        },
+        "
+        var _bar1 = require('./bar');
+        const makeX = ()=>{
+            const _bar = ()=>(0, _bar1).bar()
+            ;
+            return {
+                _bar
+            };
+        };
+        ",
+    );
+}
